import React, { useEffect, useState } from 'react'
import Slider from '../Slider'
import { styles } from './SlidingDateSelector.style'
import { SelectedDate, SlidingDateSelectorProps } from './SlidingDateSelector.types'
import Utils from './Utils'

export enum Mode {
  DAY,
  MONTH,
  YEAR,
}

const SlidingDateSelector = ({ 
    labelColor,
    maxYear,
    minYear,
    onDateUpdated,
    primaryButtonColor = '#00a989',
    primaryButtonSelectedColor = '#0092d6',
    secondaryButtonColor = '#ab3e8f'
  }:  SlidingDateSelectorProps) => {
  const [numberOfDaysInMonth, setNumberOfDaysInMonth] = useState<number>(1)
  const [selectedMode, selectMode] = useState<Mode>(Mode.DAY)
  const [selectedYear, selectYear] = useState(maxYear)
  const [selectedMonth, selectMonth] = useState(1)
  const [selectedDay, selectDay] = useState(1)

  useEffect(() => {
    setToday()
  }, [])

  const dateUpdated = (date: SelectedDate, mode?: Mode) => {
    const updatedMode = mode !== undefined ? mode : selectedMode
    onDateUpdated({
      day: updatedMode === Mode.DAY ? date.day : undefined,
      month: updatedMode === Mode.DAY || updatedMode === Mode.MONTH ? date.month : undefined,
      year: date.year 
    })
  }

  const changeDay = (day: number) => {
    selectDay(day)
    dateUpdated({ day, month: selectedMonth, year: selectedYear })
  }

  const changeMonth = (month: number) => {
    selectMonth(month)
    updateNumberOfDaysInMonth(selectedYear, month)
    dateUpdated({ day: selectedDay, month, year: selectedYear })
  }

  const changeYear = (year: number) => {
    selectYear(year)
    updateNumberOfDaysInMonth(year, selectedMonth)
    dateUpdated({ day: selectedDay, month: selectedMonth, year })
  }

  const getMonthName = (monthIn: number) => {
    let objDate = new Date()
    objDate.setDate(1)
    objDate.setMonth(monthIn - 1)

    let locale = "da-dk"
    let month = objDate.toLocaleString(locale, { month: "long" })
    return month.charAt(0).toUpperCase() + month.slice(1)
  }

  const updateNumberOfDaysInMonth = (year: number, month: number) => {
    const newNumberOfDaysInMonth = Utils.calcNumberOfDaysInMonth(year, month)
    if (selectedDay > newNumberOfDaysInMonth) {
      selectDay(newNumberOfDaysInMonth)
    }
    setNumberOfDaysInMonth(newNumberOfDaysInMonth)
  }

  const setToday = () => {
    const today = new Date()
    const year = today.getFullYear()
    const month = today.getMonth() + 1
    const day = today.getDate()
    selectDay(day)
    selectMonth(month)
    selectYear(year)
    setNumberOfDaysInMonth(Utils.calcNumberOfDaysInMonth(year, month))
    dateUpdated({ day, month, year })
  }

  const changeMode = (mode: Mode) => {
    selectMode(mode)
    dateUpdated({
      day: selectedDay,
      month: selectedMonth,
      year: selectedYear
    }, mode)
  }

  return (
    <>
      <div style={styles.container}>

        <div style={styles.buttonContainer}>
          <button
            style={{ ...styles.button, backgroundColor: selectedMode === Mode.DAY ? primaryButtonSelectedColor : primaryButtonColor }}
            onClick={() => { changeMode(Mode.DAY) }}
          >Et døgn</button>

          <button
            style={{ ...styles.button, backgroundColor: selectedMode === Mode.MONTH ? primaryButtonSelectedColor : primaryButtonColor }}
            onClick={() => { changeMode(Mode.MONTH) }}
          >En måned</button>

          <button
            style={{ ...styles.button, backgroundColor: selectedMode === Mode.YEAR ? primaryButtonSelectedColor : primaryButtonColor }}
            onClick={() => { changeMode(Mode.YEAR) }}
          >Et år</button>
        </div>
        <div style={styles.secondaryButtonContainer}>
          <button
            style={{ ...styles.button, backgroundColor: secondaryButtonColor }}
            onClick={() => {
              setToday()
              selectMode(Mode.DAY)
            }}
          >I dag</button>
        </div>
      </div>
      <>
        {
          selectedMode === Mode.DAY &&
          <Slider labelColor={labelColor} label="" valueAsString={selectedDay ? selectedDay.toString() + '.' : ''} value={selectedDay} minValue={1} maxValue={numberOfDaysInMonth} changeValue={changeDay}></Slider>
        }
        {
          (selectedMode === Mode.MONTH || selectedMode === Mode.DAY) &&
          <Slider labelColor={labelColor} label="" valueAsString={getMonthName(selectedMonth)} value={selectedMonth} minValue={1} maxValue={12} changeValue={changeMonth}></Slider>
        }
        <>
          <Slider labelColor={labelColor} label='' valueAsString={selectedYear ? selectedYear.toString() : ''} value={selectedYear} minValue={minYear} maxValue={maxYear} changeValue={changeYear}></Slider>
        </>
      </>
    </>
  )
}

export default SlidingDateSelector